//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//      LCGrid.h: --- Class related to the time serie binning
//
//      DESCRIPTION:
//             Interface to the common functionalities of a binning class.
//
//      CHANGE HISTORY:
//              AI, 2008-01-11: 1.0.0: - first version
//
//      AUTHOR:
//              A. Ibarra (AI), Wed Jan  9 17:54:04 GMT 2008
//
//

#ifndef LCGRID_H_
#define LCGRID_H_

#include <cmath>
#include <cstddef>
#include <vector>
#include "Gti.h"
#include "Dal.h"

class Grid {
 public:
  double _origin; //value at mid-point of cell zero (tick=0)
  double _stride; //width of a cell in value space
  size_t _cells;  //numbre of bins
  double _start;
  double _stop;

  Grid(double dlt=1.0, double ref=0.0) : _origin(ref), _stride(dlt) {}
  ~Grid() {}
  double operator[](double tick) const {double val = _origin + _stride * tick;
      return val > _stop ? _stop : val;  }
  double tick(double value) const {return (value - _origin)/_stride;}
  int cell( double value) const {return (int) floor(0.5+tick(value));}
};

class Span {
 public:
  size_t _min;
  size_t _max;

  Span() : _min(1), _max(0) {}
  Span(int i) : _min(i), _max(i) {}
  Span(int i0, int i1) : _min(i0), _max(i1) {}
  ~Span() {}

  bool contains( unsigned int i) const {return i >= _min && i <= _max;}

  double length() const { return _max < _min ? 0.0 : _max-_min + 1.0; }

};

class LCGrid : public Grid, public Span
{
 public:
	 LCGrid(const Grid &grid=Grid(), const Span &span=Span()) : Grid(grid), Span(span) {}
	 LCGrid (double val0, double val1, double binSize=1.0);
	 LCGrid (double val0, double val1, size_t cells=1);
	 ~LCGrid();

	 double minval() const;
	 double maxval() const;
	 double size(size_t n) const; //Return the size of the bin n
	 std::vector<double> cellLimits(size_t n) const; //return the cell range.
	 double getBinSize(size_t n) const; // return the size of the bin n.

	 void setFracExp(std::vector<double> fracExp);
	 void printFracExp();

	 void printGtiList();

	 void setGtiList(GtiList*);
	 GtiList* getGtiList();

	 void setCounts(int32*);
	 void setCounts(real64*);
	 std::vector<double> getCounts();
	 void printCounts();


	 void calFracExpWeight();
	 std::vector<double> getFracExpWeight();

	 void setCCDNumber(int ccd);
	 int getCCDNumber();

	 void calRateAndErr();

	 bool checkBinning(LCGrid* bkgLC);

  	std::vector<double> _fracExp;
  	std::vector<double> *_effBinTime;
  	GtiList *_gtiList;
	std::string _name;

  	std::vector<double> _fracExpWeight;

  	std::vector<double> _rateCorr;
	std::vector<double> _errRateCorr;
	std::vector<double> _timeStamp;
	std::vector<double> _counts;
	int _ccd;

  	void printLC();
 private:




};

std::vector<double> operator-(std::vector<double> v1,std::vector<double> v2);
std::vector<double> operator+(std::vector<double> v1,std::vector<double> v2);
std::vector<double> operator*(std::vector<double> v1,std::vector<double> v2);
std::vector<double> operator/(std::vector<double> v1,std::vector<double> v2);

std::vector<double> operator*(std::vector<double> v1, double n);
#endif /*LCGRID_H_*/

